
#include <Arduino_JSON.h>
#include <ESP8266WiFi.h>
#include <MQTT.h>

#define LED_PIN LED_BUILTIN //LED_BUILTIN is built in LED

#include "DHTesp.h"

#ifdef ESP32
#pragma message(THIS EXAMPLE IS FOR ESP8266 ONLY!)
#error Select ESP8266 board.
#endif


WiFiClient net;
MQTTClient MQTTclient; 


const char ssid[] = "?????";
const char pass[] = "?????";

// the IP address for the MQTT server
char MQTTip[] = "192.168.1.71";   

unsigned long UpdateCount = 6;
unsigned long TickCount = UpdateCount-1;
unsigned long PacketCount = 0;
unsigned long lastMillis = 0;
unsigned long roundRobin = 0;


DHTesp dht[5] = {DHTesp(), DHTesp(), DHTesp(), DHTesp(), DHTesp()};

float temps[5] = {0, };
float hums[5] = {0, };

void setup() {

  pinMode(LED_PIN, OUTPUT);

  dht[0].setup(4, DHTesp::DHT22);
  dht[1].setup(16, DHTesp::DHT22);
  dht[2].setup(5, DHTesp::DHT22);
  dht[3].setup(14, DHTesp::DHT22);
  dht[4].setup(12, DHTesp::DHT22);
 
  Serial.begin(115200);

  WiFi.begin(ssid, pass);

  // Note: Local domain names (e.g. "Computer.local" on OSX) are not supported by Arduino.
  // You need to set the IP address directly.
  MQTTclient.begin(MQTTip, net);
  MQTTclient.onMessage(messageReceived);

  WiFiconnect();
  
  MQTTclient.publish("/ervdas/status", "start");

    

  MQTTclient.publish("/ervdas/status", "run");
}

void loop() {
  
  MQTTclient.loop();
  
  delay(10);  // <- fixes some issues with WiFi stability

  if (!MQTTclient.connected()) {
    WiFiconnect();
  }

  float temp = 0;
  float hum = 0;
  
  switch(roundRobin)
  {
    case 0:
      temp = dht[roundRobin].getTemperature();
      hum = dht[roundRobin].getHumidity();
      break;
    case 1:
      temp = dht[roundRobin].getTemperature();
      hum = dht[roundRobin].getHumidity();
      break;
    case 2:
      temp = dht[roundRobin].getTemperature();
      hum = dht[roundRobin].getHumidity();
      break;
    case 3:
      temp = dht[roundRobin].getTemperature();
      hum = dht[roundRobin].getHumidity();
      break;
    case 4:
      temp = dht[roundRobin].getTemperature();
      hum = dht[roundRobin].getHumidity();
      break;
    
  }

  if(!isnan(temp) ) 
    temps[roundRobin] = temp;
  else
    temps[roundRobin] = 999;

  if(!isnan(hum)) 
    hums[roundRobin] = hum;
  else
    hums[roundRobin] = 999;

    
  roundRobin++;
  if(roundRobin == 5)
  {
    roundRobin = 0;
    SendMQTT();
  }

    
    /*
  // publish a message roughly every second.
  if (millis() - lastMillis > (UpdateCount * 1000)) 
  { 
     lastMillis = millis();
     SendMQTT();
  }
  */
  delay(500);

}


void WiFiconnect() {
  
  Serial.print("checking wifi...");
  Serial.println(ssid);
  
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
  }
  
  Serial.println("\nWiFi Connected! ");
  Serial.println( WiFi.localIP());

  Serial.print("\nMQTT connecting...");
  while (!MQTTclient.connect(WiFi.localIP().toString().c_str() , "try", "try")) {
    Serial.print(".");
    delay(800);
    digitalWrite(LED_PIN, LOW);
    delay(200);
    digitalWrite(LED_PIN, HIGH);
  }

  Serial.println("\nMQTT Connected!");

  digitalWrite(LED_PIN, LOW);
  
  MQTTclient.subscribe("/ervdas/control");  
  MQTTclient.publish("/ervdas/status", "start");
  MQTTclient.publish("/ervdas/localip", "/ervdas/localip/" + WiFi.localIP().toString());

}

void SendMQTT()
{
  digitalWrite(LED_PIN, LOW);
    
  JSONVar myArray;

  
  
  //STALE
  myArray[0] = getTemperature(0);
  myArray[1] = getHumidity(0);
 

  //FRESH
  myArray[2] = getTemperature(1);
  myArray[3] = getHumidity(1);
  

  //INLET
  myArray[4] = getTemperature(2);
  myArray[5] = getHumidity(2);

  
  //EXHAUST
  myArray[6] = getTemperature(3);
  myArray[7] = getHumidity(3);
  
  
  //LOFT
  myArray[8] = getTemperature(4);
  myArray[9] = getHumidity(4); 

  
  //MISC
  myArray[10] = (PacketCount++);
  myArray[11] = WiFi.RSSI();
  myArray[12] = analogRead(0);
  
   
  String jsonString = JSON.stringify(myArray);
  
  MQTTclient.publish("/ervdas/sensors", jsonString);
  Serial.print("/ervdas/sensors " );
  Serial.print(jsonString );
  
  Serial.println();

  digitalWrite(LED_PIN, HIGH);
}

float getTemperature(int i)
{
  float val = temps[i];
  
  if(val != 999)
    return toFixed(val);
  else
    return 9999;
}

float getHumidity(int i)
{
  float val = hums[i];
 
  if(val != 999)
    return toFixed(val);
  else
    return 9999;
}

float toFixed(float x)
{
 
  return (float)((long)(x * 10.0f));
  
  //char buf[40] = {0, };
  //sprintf(buf, "%2.2f", f);
  //return atof(buf);
}

void messageReceived(String &topic, String &payload) {

  digitalWrite(LED_PIN, LOW);
  
  Serial.println("incoming: " + topic + " - " + payload);
  
  if(topic == "/ervdas/control")
  {  
    if(payload.startsWith("UpdateCount"))
    {
      UpdateCount = (long)payload.substring(12).toInt();
      TickCount = UpdateCount -1;
    }
    else if(payload.startsWith("read"))
    {
      SendMQTT();
       MQTTclient.publish("/ervdas/localip", "/ervdas/localip/" + WiFi.localIP().toString());
    }
  
    MQTTclient.publish("/ervdas/payload", topic + "/" + payload);

  }

  digitalWrite(LED_PIN, HIGH);
 
}
